//+------------------------------------------------------------------+
//|                                                      NextBar.mq5 |
//|                        Copyright 2020, MetaQuotes Software Corp. |
//|                            https://www.mql5.com/ru/articles/7535 |
//+------------------------------------------------------------------+
#property copyright "Copyright 2020, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.0"


//#include <MT4Bridge/MT4Orders.mqh>
//#include <MT4Bridge/MT4Time.mqh>
#include <Marketeer/Converter.mqh>
#include <OLAP/SingleNextBarDemo.mqh>


input string PositiveHours = "";
input string NegativeHours = "";
input double Lot = 0.1;
input uint ActiveDayOfWeek = 0; // ActiveDayOfWeek (1-mon, 2-tue... 5-fri)


datetime lastBar = 0;

int trendHours[], reverseHours[];
int singleStrategy;


int MathSign(const double x)
{
  return x > 0 ? +1 : (x < 0 ? -1 : 0);
}

int OnInit()
{
  lastBar = 0;
  
  const int trend = parseHours(PositiveHours, trendHours);
  const int reverse = parseHours(NegativeHours, reverseHours);
  singleStrategy = MathSign(trend) - MathSign(reverse);
  
  return trend > 0 || reverse > 0 ? INIT_SUCCEEDED : INIT_PARAMETERS_INCORRECT;
}

double plus = 0;
double minus = 0;
int countadd = 0;
int countsub = 0;

void OnDeinit(const int reason)
{
  Converter<ulong,double> converter;
  const double infinity = converter[0x7FF0000000000000];
  if(!singleStrategy)
  {
    Print("Warning! Mixed strategy used, stats are merged (not informative)");
  }
  else
  if(StringLen(NegativeHours) > 0)
  {
    Print("Reversal strategy is enabled: PF and WIN% are inversed to actual values");
  }
  Print("PF: ", (minus > 0 ? plus / minus : (plus > 0 ? infinity : 0)), ", WIN%: ", (countadd + countsub > 0 ? (float)(100.0 * countadd / (countadd + countsub)) : 0));
}

bool isNewBar()
{
  if(lastBar != iTime(_Symbol, 0, 0))
  {
    lastBar = iTime(_Symbol, 0, 0);
    return true;
  }
  return false;
}


void OnTick()
{
  if(!isNewBar()) return;

  const int h = TimeHour(lastBar);

  int mode = 0;
  
  if(ArrayFind(trendHours, h) > -1)
  {
    mode = +1;
  }
  else
  if(ArrayFind(reverseHours, h) > -1)
  {
    mode = -1;
  }

  if(ActiveDayOfWeek != 0 && ActiveDayOfWeek != TimeDayOfWeek(lastBar)) mode = 0; // skip all days except specified

  // pick up existing orders (if any)
  const int direction = CurrentOrderDirection();
  
  if(mode == 0)
  {
    if(direction != 0)
    {
      double tradedBar = iOpen(_Symbol, _Period, 0) - iOpen(_Symbol, _Period, 1);
      double previousBar = iOpen(_Symbol, _Period, 1) - iOpen(_Symbol, _Period, 2);
      
      //tradedBar = tradedBar + MathSign(tradedBar) * -1 * iSpread(_Symbol, _Period, 1) * _Point;
      //previousBar = previousBar + MathSign(previousBar) * -1 * iSpread(_Symbol, _Period, 2) * _Point;
      const double spread = Ask - Bid; // MathMax(MathMax(iSpread(_Symbol, _Period, 1), iSpread(_Symbol, _Period, 2)), iSpread(_Symbol, _Period, 0)) * _Point;
      
      double mult = tradedBar * previousBar;
      // in trend mode positive value is ok, negative is bad
      // in reverse mode negative value is ok, positive is bad
      double value = MathSign(mult) * MathAbs(tradedBar);
      
      // accuracy with calculations is limited by the following factors:
      // MT reports only minimal spread per bar, thereas trades are occured usually with larger spreads
      // other bar spread affects every sell trades, thereas in OLAP stats we use current bar spread (can be extended)
      //   for sells spread is virtually 'charged' at close time (next open ask), for buys - at open time (this open ask)
      // spreads are subtracted from profits but added to losses
      //   so in the following 2 conditional paths spreads should be counted differently,
      //   depending from strategy (trend or reverse)
      
      string status;
      
      //value += MathSign(value) * -1 * spread;
      double fix = 0;

      if(value >= 0)
      {
        // trend profits
        // reverse losses

        fix = value - singleStrategy * spread;
        //plus += value - spread;
        value = fix;
        plus += value;
        
        countadd++;
        if(OrderProfit() < 0)
        {
          status = "-trend";
        }
        else
        {
          status = "+trend";
        }
      }
      else
      {
        // reverse profits
        // trend losses

        fix = -value + singleStrategy * spread;
        //fix = -fix;
        //minus += -value + spread;
        value = -fix;
        
        //minus -= value - spread;
        minus -= value;
        
        countsub++;
        if(OrderProfit() > 0)
        {
          status = "+reverse";
        }
        else
        {
          status = "-reverse";
        }
      }
      
      /* debug
      MqlTick tick[1];
      SymbolInfoTick(_Symbol, tick[0]);
      ArrayPrint(tick);
      Print(iSpread(_Symbol, _Period, 1), " ", iSpread(_Symbol, _Period, 2), " ", (float)fix);
      */
      
      //Print("Status? ", OrderClosePrice(), " ", OrderOpenPrice(), " ", (OrderClosePrice() - OrderOpenPrice()), " ", (float)(OrderClosePrice() - OrderOpenPrice()), " ", ((float)(OrderClosePrice() - OrderOpenPrice())) / (float)_Point);
      const int orderPoints = (OrderType() == OP_SELL ? -1 : +1) * (int)(((float)(OrderClosePrice() - OrderOpenPrice())) / (float)_Point); //  * (OrderType() == OP_BUY ? +1 : -1)
      const int barPoints = (int)((float)tradedBar / (float)_Point);
      const int prevPoints = (int)((float)previousBar / (float)_Point);
      Print("Status: ", status, " ", (OrderType() == OP_BUY ? "BUY" : "SELL"), " ", (float)value, ShortToString(0xB1), (float)spread, " Order: ", orderPoints, " ", (float)OrderClosePrice(), " ", (float)OrderOpenPrice(),
            " Bar: ", barPoints, " ", (float)iOpen(_Symbol, _Period, 0), " ", (float)iOpen(_Symbol, _Period, 1), " Prev:", prevPoints);

      OrdersCloseAll(); // if any
    }
    return;
  }
  
  const double previousChange = iOpen(_Symbol, _Period, 0) - iOpen(_Symbol, _Period, 1);
  if(MathAbs(previousChange) <= Ask - Bid /*MathMax(iSpread(_Symbol, _Period, 1), iSpread(_Symbol, _Period, 0)) * _Point*/) return;

  const int previousDirection = previousChange > 0 ? +1 : (previousChange < 0 ? -1 : 0);
  
  if(previousDirection == 0) return; // undefined bar
  
  if(direction != 0) // there exist open orders
  {
    if(mode == +1) // keep direction
    {
      if(direction == previousDirection) return; // existing trade goes no
    }
    else // -1
    {
      if(direction == -previousDirection) return; // existing trade goes no
    }
    OrdersCloseAll();
  }
  
  
  const int type = previousDirection * mode > 0 ? OP_BUY : OP_SELL;
  
  const double p = type == OP_BUY ? SymbolInfoDouble(_Symbol, SYMBOL_ASK) : SymbolInfoDouble(_Symbol, SYMBOL_BID);
    
  OrderSend(_Symbol, type, Lot, p, 100, 0, 0);
}
