//+------------------------------------------------------------------+
//|                                                     OLAPRPRT.mq5 |
//|                                 Copyright © 2016-2020, Marketeer |
//|                          https://www.mql5.com/en/users/marketeer |
//|               Online Analytical Processing of trading hypercubes |
//|                            https://www.mql5.com/ru/articles/6602 |
//|                            https://www.mql5.com/ru/articles/6603 |
//|                            https://www.mql5.com/ru/articles/7535 |
//+------------------------------------------------------------------+

#property copyright "Copyright © 2016-2020, Marketeer"
#property link      "https://www.mql5.com/ru/articles/6602"
#property version   "1.0"
#property description "Trade History Analysis (online, tester and account HTML-reports, CSV-reports): Non-trading Expert Adviser demonstrating OLAP classes"

#property strict

#include <OLAP/OLAPTrades.mqh>
#include <OLAP/HTMLcube.mqh>
#include <OLAP/CSVcube.mqh>
#include <Marketeer/GroupSettings.mqh>


input GroupSettings OLAP_Settings; // O L A P     S E T T I N G S

sinput string X = "————— X axis —————"; // · X ·
input SELECTORS SelectorX = DEFAULT_SELECTOR_TYPE; // · SelectorX
input ENUM_FIELDS FieldX = DEFAULT_SELECTOR_FIELD /* field does matter only for some selectors */; // · FieldX

sinput string Y = "————— Y axis —————"; // · Y ·
input SELECTORS SelectorY = SELECTOR_NONE; // · SelectorY
input ENUM_FIELDS FieldY = FIELD_NONE; // · FieldY

sinput string Z = "————— Z axis —————"; // · Z ·
input SELECTORS SelectorZ = SELECTOR_NONE; // · SelectorZ
input ENUM_FIELDS FieldZ = FIELD_NONE; // · FieldZ

sinput string F = "————— Filter —————"; // · F ·
input SELECTORS _Filter1 = SELECTOR_NONE; // · Filter1
input ENUM_FIELDS _Filter1Field = FIELD_NONE; // · Filter1Field
input double _Filter1value1 = 0; // · Filter1value1
input double _Filter1value2 = 0; // · Filter1value2

sinput string A = "————— Aggregator —————"; // · A ·
input AGGREGATORS _AggregatorType = DEFAULT_AGGREGATOR_TYPE; // · AggregatorType
input ENUM_FIELDS _AggregatorField = DEFAULT_AGGREGATOR_FIELD; // · AggregatorField
input SORT_BY _SortBy = SORT_BY_NONE; // · SortBy


SELECTORS _selectorArray[4];
ENUM_FIELDS _selectorField[4];


LogDisplay _display(10, 3);


int OnInit()
{
  _selectorArray[0] = SelectorX;
  _selectorArray[1] = SelectorY;
  _selectorArray[2] = SelectorZ;
  _selectorArray[3] = _Filter1;
  _selectorField[0] = FieldX;
  _selectorField[1] = FieldY;
  _selectorField[2] = FieldZ;
  _selectorField[3] = _Filter1Field;
  
  if(ReportFile == "")
  {
    Print("Analyzing account history");
    _defaultEngine.setAdapter(&_defaultHistoryAdapter);
  }
  else
  {
    if(StringFind(ReportFile, ".htm") > 0 && _defaultHTMLReportAdapter.load(ReportFile))
    {
      _defaultEngine.setAdapter(&_defaultHTMLReportAdapter);
    }
    else
    if(StringFind(ReportFile, ".csv") > 0 && _defaultCSVReportAdapter.load(ReportFile))
    {
      _defaultEngine.setAdapter(&_defaultCSVReportAdapter);
    }
    else
    {
      Print("Unknown file format: ", ReportFile);
      return INIT_PARAMETERS_INCORRECT;
    }
  }

  EventSetTimer(1);
  return INIT_SUCCEEDED;
}

void OnTimer()
{
  EventKillTimer();
  _defaultEngine.process(_selectorArray, _selectorField,
        _AggregatorType, _AggregatorField,
        _display,
        _SortBy,
        _Filter1value1, _Filter1value2);
}


void OnTick()
{
}
